using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.Platform.Windows;

namespace SpotLight
{
    public partial class SpotLightForm : Form
    {
        #region Attributes
        private Icon newIcon;

        public int viewportW = 800, viewportH = 600;
        public double winX = 5.0;
        public double winY = 5.0;
        //public double winZ=10.0;
        public Rectangle rec;
        public bool fog = false;

        float[] lightPos ={ 0.0f, 1.2f, 0.0f, 1.0f };//{ -1.5f, 3.0f, 4.5f, 1.0f };//{ -3.8f, 3.8f, 3.8f, 0.0f };//
        float[] spotDirection ={ -1.0f,-1.0f,-1.0f};//{ -0.8f, 3.0f, 6.0f, 1.0f };
        private double theta=0.0;
        //private float[] lightPos ={ 1.5f, 1.5f, 2.5f, 1.0f };
        //light colors
        private float[] amb ={ 0.2f, 0.2f, 0.2f, 1.0f };
        private float[] diff ={ 1.5f, 0.173f, 0.115f, 1.0f };
        private float[] spec ={ 0.5f, 0.5f, 0.5f, 1.0f };
        #endregion
        public SpotLightForm()
        {
            InitializeComponent();
            glControl.InitializeContexts();
            initializeOpenGL();
            this.SetBounds(0, 0, viewportW, viewportH);
            getIconFromBitmap();
        }
        public void getIconFromBitmap()
        {
            Bitmap myBitmap = new Bitmap(@"..\..\Images\rose12.jpg");
            IntPtr Hicon = myBitmap.GetHicon();
            newIcon = Icon.FromHandle(Hicon);
            this.Icon = newIcon;
            //DestroyIcon(newIcon.Handle);
        }
        private void initializeOpenGL()
        {
            Gl.glViewport(0, 0, viewportW, viewportH);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();

            Gl.glOrtho(-winX * (8.0 / 6.0), winX * (8.0 / 6.0), -winY, winY, -10, 40);
            //Glu.gluPerspective(90, 1.4, 0.1, 10);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            //Glu.gluLookAt(2.0, 1.0, 2.0, 0, 0, 0, 0, 1, 0);
            Glu.gluLookAt(1.0, 1.0, 2.0, 0, 0, 0, 0, 1, 0);


            Gl.glClearColor(0, 0, 0, 1);
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glShadeModel(Gl.GL_SMOOTH);
            initLights();
            Gl.glEnable(Gl.GL_LIGHTING);
            //initFog();
        }       
        private void initLights()
        {
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, lightPos);

            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, amb);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diff);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, spec);
            Gl.glLighti(Gl.GL_LIGHT0, Gl.GL_SPOT_CUTOFF, 30);//20
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPOT_DIRECTION, spotDirection);
            Gl.glLightf(Gl.GL_LIGHT0, Gl.GL_SPOT_EXPONENT, 1.5f);

            Gl.glLightf(Gl.GL_LIGHT0, Gl.GL_CONSTANT_ATTENUATION, 1.0f);
            Gl.glLightf(Gl.GL_LIGHT0, Gl.GL_LINEAR_ATTENUATION, 0.2f);
            Gl.glLightf(Gl.GL_LIGHT0, Gl.GL_QUADRATIC_ATTENUATION, 0.1f);

            //Gl.glEnable(Gl.GL_LIGHT1);         
            Gl.glEnable(Gl.GL_LIGHT0);
         
        }
        private void initFog()
        {
            Gl.glEnable(Gl.GL_FOG);
            
                float[] fogColor ={ 0.1f, 0.1f, 0.15f, 1.0f };
                int FogMode = Gl.GL_EXP;//GL_EXP2;//GL_LINEAR;//GL_EXP;
                Gl.glFogi(Gl.GL_FOG_MODE, FogMode);
                Gl.glFogfv(Gl.GL_FOG_COLOR, fogColor);
                Gl.glFogf(Gl.GL_FOG_DENSITY, 0.2f);
                Gl.glHint(Gl.GL_FOG_HINT, Gl.GL_NICEST);//GL_FASTEST);//GL_DONT_CARE);//GL_NICEST
                Gl.glFogf(Gl.GL_FOG_START, 3.5f);
                Gl.glFogf(Gl.GL_FOG_END, 5.0f);
        
            Gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        }
        private void glControl_Paint(object sender, PaintEventArgs e)
        {            
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            for (double a = -5.0; a < 5.0; a += 0.5)
            {
                for (double b = -5.00; b < 5.0; b += 0.5)
                {
                    Gl.glPushMatrix();
                    Gl.glTranslated(a, 0, b);
                    Sphere(0.25, 10, 10);
                    Gl.glPopMatrix();
                }
            }
           
                spotDirection[0] = (float)Math.Cos(theta);
                spotDirection[2] = (float)Math.Sin(theta);
                Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPOT_DIRECTION, spotDirection);
                theta += 0.05;
                glControl.Invalidate();            
        }
        private void glControl_Resize(object sender, EventArgs e)
        {
            rec = ClientRectangle;
            viewportW = rec.Width;
            viewportH = rec.Height;

            Gl.glViewport(0, 0, viewportW, viewportH);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            //Glu.gluPerspective(60, 1.4, 0.1, 10);
            if (viewportW > viewportH)
                Gl.glOrtho(-winX * viewportW / viewportH, winX * viewportW / viewportH, -winY, winY, -10, 10);
            else
                Gl.glOrtho(-winX, winX, -winY * viewportH / viewportW, winY * viewportH / viewportW, -10, 10);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(2.0, 1.0, 2.0, 0, 0, 0, 0, 1, 0);
            glControl.Invalidate();
        }       
        private void glControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                Application.Exit();
        }
        #region 3D Objects
        public void Sphere(double radius, int slices, int stacks)
        {
            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
            Glu.gluSphere(quad, radius, slices, stacks);
        }
        public void Disk(double inRadius, double outRadius, int slices, int stacks)
        {
            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
            Glu.gluDisk(quad, inRadius, outRadius, slices, stacks);
        }
        public void Cylinder(double baseRadius, double topRadius, double height, int slices, int stacks)
        {
            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_FLAT);//SMOOTH);
            Glu.gluCylinder(quad, baseRadius, topRadius, height, slices, stacks);
        }
        public void Cone(double baseRadius, double height, int slices, int stacks)
        {
            Glu.GLUquadric quad = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
            Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
            Glu.gluCylinder(quad, baseRadius, 0.0, height, slices, stacks);
        }
        public void drawCube(double side)
        {
            double hs = side / 2.0;
            double[,] vertex = {{hs,hs,hs},
                                {hs,hs,-hs},
                                {-hs,hs,-hs},
                                {-hs,hs,hs},
                                {hs,-hs,hs},
                                {hs,-hs,-hs},
                                {-hs,-hs,-hs},
                                {-hs,-hs,hs}};
            int[,] face ={  {0,1,2,3},
                            {7,6,5,4},           
                            {0,4,5,1},
                            {6,7,3,2},
                            {0,3,7,4},
                            {2,1,5,6}};
            double[,] normal ={ {0,1,0},
                                {0,-1,0},
                                {1,0,0},
                                {-1,0,0},
                                {0,0,1},
                                {0,0,-1}};
            Gl.glBegin(Gl.GL_QUADS);
            Gl.glColor4f(0.11f, 0.21f, 0.65f, 1.0f);
            for (int f = 0; f < 6; f++)
            {
                for (int v = 0; v < 4; v++)
                {
                    Gl.glNormal3d(normal[f, 0], normal[f, 1], normal[f, 2]);
                    Gl.glVertex3d(vertex[face[f, v], 0], vertex[face[f, v], 1], vertex[face[f, v], 2]);
                }
            }
            Gl.glEnd();
        }
        #endregion
    }
}